"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const useError_1 = __importDefault(require("./useError"));
// todo test if it make sense to merge useState into one object.
function useAsyncHandler(onAction, options) {
    const { isRetryAllowed, strict } = options || {};
    const [isLoading, setIsLoading] = (0, react_1.useState)(false);
    const [isDone, setIsDone] = (0, react_1.useState)(false);
    const [data, setData] = (0, react_1.useState)(null);
    const { error, setError } = (0, useError_1.default)(options);
    const handleActionMemoized = (0, react_1.useCallback)((actionParam) => __awaiter(this, void 0, void 0, function* () {
        if (isDone && !isRetryAllowed && strict) {
            console.warn('Action is blocked because isDone is true. Async action is ignored. Possible leak detected.');
            return null;
        }
        if (isLoading) {
            console.warn('action is loading already. Ignore rerunning actions.');
            return null;
        }
        try {
            setIsLoading(true);
            if (error)
                setError('');
            const result = yield onAction(actionParam);
            setData(result);
            setIsDone(true);
            setIsLoading(false);
            return result;
        }
        catch (e) {
            setError(e);
            setIsLoading(false);
        }
    }), [isDone, error, isLoading, onAction, isRetryAllowed, strict]);
    const reset = (0, react_1.useCallback)(function reset() {
        if (isLoading) {
            // why it is not firing? Even if loading is true already
            console.warn('You are using .reset() during active action. Some data can be overwritten');
        }
        setIsDone(false);
        setIsLoading(false);
        setError(null);
        setData(null);
    }, [isLoading]);
    return {
        execute: handleActionMemoized,
        isLoading,
        isDone,
        error,
        reset,
        data,
    };
}
exports.default = useAsyncHandler;
//# sourceMappingURL=useAsyncHandler.js.map